package gregapi.code;

import java.util.ArrayList;

/**
 * @author Gregorius Techneticies
 * 
 * Useful for tagging things. It could Tag anything.
 * Better than Strings for tagging Stuff since you can do an == Check rather than needing to use equals.
 */
public final class TagData implements ICondition<ITagDataContainer> {
	private static final ArrayList<TagData> TAGS = new ArrayListNoNulls<TagData>();
	
	public final int mSubtagID;
	public final String mName;
	
	private TagData(String aName) {
		mSubtagID = TAGS.size();
		mName = aName;
		TAGS.add(this);
	}
	
	public static TagData createTagData(String aName) {
		aName = aName.toUpperCase();
		for (TagData tSubTag : TAGS) if (tSubTag.mName.equals(aName)) return tSubTag;
		return new TagData(aName);
	}
	
	@Override
	public String toString() {
		return mName;
	}
	
	@Override
	public boolean isTrue(ITagDataContainer aObject) {
		return aObject.contains(this);
	}
	
	@Override
	public boolean equals(Object aObject) {
		return (aObject instanceof TagData && ((TagData)aObject).mSubtagID == mSubtagID);
	}
	
	@Override
	public int hashCode() {
		return mSubtagID;
	}
}