package gregapi.item;

import static gregapi.data.CS.RA;
import static gregapi.data.CS.RES_PATH_ITEM;
import static gregapi.data.CS.SHOW_HIDDEN_ITEMS;
import gregapi.GT_API;
import gregapi.code.ArrayListNoNulls;
import gregapi.data.CS.ModIDs;
import gregapi.data.IL;
import gregapi.data.TC.TC_AspectStack;
import gregapi.lang.LanguageHandler;
import gregapi.old.SubTag;
import gregapi.old.interfaces.IFoodStat;
import gregapi.old.interfaces.IItemBehaviour;
import gregapi.old.interfaces.IItemContainer;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictManager;
import gregapi.util.OM;
import gregapi.util.UT;

import java.util.Arrays;
import java.util.BitSet;
import java.util.HashMap;
import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.EnumAction;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;
import squeek.applecore.api.food.FoodValues;
import squeek.applecore.api.food.IEdible;
import cpw.mods.fml.common.Loader;
import cpw.mods.fml.common.Optional;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 * 
 * For Custom Items.
 */
@Optional.InterfaceList(value = {
  @Optional.Interface(iface = "squeek.applecore.api.food.IEdible", modid = ModIDs.APC)
})
public class ItemMetaRandom extends ItemMeta implements IEdible {
	public final BitSet mEnabledItems = new BitSet(32767);
	public final BitSet mVisibleItems = new BitSet(32767);
	public final IIcon[][] mIconList = new IIcon[32767][1];
	
	public final HashMap<Short, IFoodStat> mFoodStats = new HashMap<Short, IFoodStat>();
	public final HashMap<Short, Long[]> mElectricStats = new HashMap<Short, Long[]>();
	public final HashMap<Short, Short> mBurnValues = new HashMap<Short, Short>();
	
	/**
	 * Creates the Item using these Parameters.
	 * @param aUnlocalized The unlocalised Name of this Item. DO NOT START YOUR UNLOCALISED NAME WITH "gt."!!!
	 */
	public ItemMetaRandom(String aUnlocalized) {
		super(aUnlocalized);
	}
	
    @Override
	public ItemStack onItemRightClick(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
    	use(aStack, 0, aPlayer);
		isItemStackUsable(aStack);
    	IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
    	if (tStat != null && aPlayer.canEat(tStat.alwaysEdible(this, aStack, aPlayer))) aPlayer.setItemInUse(aStack, 32);
    	return super.onItemRightClick(aStack, aWorld, aPlayer);
    }
    
	/**
	 * This adds a Custom Item.
	 * @param aID The Id of the assigned Item [0 - 32766]
	 * @param aEnglish The Default Localised Name of the created Item
	 * @param aToolTip The Default ToolTip of the created Item, you can also insert null for having no ToolTip
	 * @param aFoodBehavior The Food Value of this Item. Can be null aswell. Just a convenience thing.
	 * @param aRandomData The OreDict Names you want to give the Item. Also used for TC Aspects and some other things.
	 * @return An ItemStack containing the newly created Item.
	 */
	public final ItemStack addItem(int aID, String aEnglish, String aToolTip, Object... aRandomData) {
		if (aToolTip == null) aToolTip = "";
		if (aID >= 0 && aID < 32767) {
			ItemStack rStack = new ItemStack(this, 1, aID);
			mEnabledItems.set(aID);
			mVisibleItems.set(aID);
			LanguageHandler.add(getUnlocalizedName(rStack) + ".name", aEnglish);
			LanguageHandler.add(getUnlocalizedName(rStack) + ".tooltip", aToolTip);
			List<TC_AspectStack> tAspects = new ArrayListNoNulls<TC_AspectStack>();
			// Important Stuff to do first
			for (Object tRandomData : aRandomData) if (tRandomData instanceof SubTag) {
				if (tRandomData == SubTag.INVISIBLE) {
					mVisibleItems.set(aID, false);
					continue;
				}
				if (tRandomData == SubTag.NO_UNIFICATION) {
					OM.blacklist(rStack);
					continue;
				}
			}
			// now check for the rest
			for (Object tRandomData : aRandomData) if (tRandomData != null) {
				boolean tUseOreDict = true;
				if (tRandomData instanceof IFoodStat) {
					setFoodBehavior(aID, (IFoodStat)tRandomData);
					if (((IFoodStat)tRandomData).getFoodAction(this, rStack) == EnumAction.eat) {
						int tFoodValue = ((IFoodStat)tRandomData).getFoodLevel(this, rStack, null);
				        if (tFoodValue > 0) RA.addCannerRecipe(rStack, IL.IC2_Food_Can_Empty.get(tFoodValue), ((IFoodStat)tRandomData).isRotten(this, rStack, null)?IL.IC2_Food_Can_Spoiled.get(tFoodValue):IL.IC2_Food_Can_Filled.get(tFoodValue), null, tFoodValue*100, 1);
					}
					tUseOreDict = false;
				}
				if (tRandomData instanceof IItemBehaviour) {
					addItemBehavior(aID, (IItemBehaviour<ItemMeta>)tRandomData);
					tUseOreDict = false;
				}
				if (tRandomData instanceof IItemContainer) {
					((IItemContainer)tRandomData).set(rStack);
					tUseOreDict = false;
				}
				if (tRandomData instanceof SubTag) {
					continue;
				}
				if (tRandomData instanceof TC_AspectStack) {
					((TC_AspectStack)tRandomData).addToAspectList(tAspects);
					continue;
				}
				if (tRandomData instanceof OreDictItemData) {
					if (UT.Code.stringValid(tRandomData)) OreDictManager.INSTANCE.registerOre(tRandomData, rStack); else OM.data(rStack, (OreDictItemData)tRandomData);
					continue;
				}
				if (tUseOreDict) {
					OreDictManager.INSTANCE.registerOre(tRandomData, rStack);
					continue;
				}
			}
			if (GT_API.instance.mThaumcraftCompat != null) GT_API.instance.mThaumcraftCompat.registerThaumcraftAspectsToItem(rStack, tAspects, false);
			return rStack;
		}
		return null;
	}
	
	/**
	 * Sets a Food Behavior for the Item.
	 * 
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32765]
	 * @param aFoodBehavior the Food Behavior you want to add.
	 * @return the Item itself for convenience in constructing.
	 */
	public ItemMetaRandom setFoodBehavior(int aMetaValue, IFoodStat aFoodBehavior) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length()) return this;
		if (aFoodBehavior == null) mFoodStats.remove((short)aMetaValue); else mFoodStats.put((short)aMetaValue, aFoodBehavior);
		return this;
	}
	
	/**
	 * Sets the Furnace Burn Value for the Item.
	 * 
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32765]
	 * @param aValue 200 = 1 Burn Process = 500 EU, max = 32767 (that is 81917.5 EU)
	 * @return the Item itself for convenience in constructing.
	 */
	public ItemMetaRandom setBurnValue(int aMetaValue, int aValue) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length() || aValue < 0) return this;
		if (aValue == 0) mBurnValues.remove((short)aMetaValue); else mBurnValues.put((short)aMetaValue, aValue>Short.MAX_VALUE?Short.MAX_VALUE:(short)aValue);
		return this;
	}
	
	/**
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32765]
	 * @param aMaxCharge Maximum Charge. (if this is == 0 it will remove the Electric Behavior)
	 * @param aTransferLimit Transfer Limit.
	 * @param aTier The electric Tier.
	 * @param aSpecialData If this Item has a Fixed Charge, like a SingleUse Battery (if > 0).
	 * Use -1 if you want to make this Battery chargeable (the use and canUse Functions will still discharge if you just use this)
	 * Use -2 if you want to make this Battery dischargeable.
	 * Use -3 if you want to make this Battery charge/discharge-able.
	 * @return the Item itself for convenience in constructing.
	 */
	public ItemMetaRandom setElectricStats(int aMetaValue, long aMaxCharge, long aTransferLimit, long aTier, long aSpecialData, boolean aUseAnimations) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length()) return this;
		if (aMaxCharge == 0) mElectricStats.remove((short)aMetaValue); else {
			mElectricStats.put((short)aMetaValue, new Long[] {aMaxCharge, Math.max(0, aTransferLimit), Math.max(-1, aTier), aSpecialData});
			if (aUseAnimations) mIconList[aMetaValue] = Arrays.copyOf(mIconList[aMetaValue], Math.max(9, mIconList[aMetaValue].length));
		}
		return this;
	}
	
	/**
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32765]
	 * @param aMaxCharge Maximum Charge. (if this is == 0 it will remove the Electric Behavior)
	 * @param aTransferLimit Transfer Limit.
	 * @param aTier The electric Tier.
	 * @param aSpecialData If this Item has a Fixed Charge, like a SingleUse Battery (if > 0).
	 * Use -1 if you want to make this Battery chargeable (the use and canUse Functions will still discharge if you just use this)
	 * Use -2 if you want to make this Battery dischargeable.
	 * Use -3 if you want to make this Battery charge/discharge-able.
	 * @return the Item itself for convenience in constructing.
	 */
	public ItemMetaRandom setFluidContainerStats(int aMetaValue, long aCapacity, long aStacksize) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length()) return this;
		if (aCapacity < 0) mElectricStats.remove((short)aMetaValue); else mFluidContainerStats.put((short)aMetaValue, new Long[] {aCapacity, Math.max(1, aStacksize)});
		return this;
	}
	
    @Override
    public int getMaxItemUseDuration(ItemStack aStack) {
        return mFoodStats.get((short)getDamage(aStack)) == null ? 0 : 32;
    }
    
    @Override
	public EnumAction getItemUseAction(ItemStack aStack) {
    	IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
        return tStat == null ? EnumAction.none : tStat.getFoodAction(this, aStack);
    }
    
    @Override
	public ItemStack onEaten(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
    	IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
    	if (tStat != null) {
    		if (Loader.isModLoaded(ModIDs.APC)) {
    			aPlayer.getFoodStats().func_151686_a((ItemFood)UT.Reflection.callConstructor("squeek.applecore.api.food.ItemFoodProxy.ItemFoodProxy", 0, null, true, this), aStack);
    		} else {
    			aPlayer.getFoodStats().addStats(tStat.getFoodLevel(this, aStack, aPlayer), tStat.getSaturation(this, aStack, aPlayer));
    		}
            tStat.onEaten(this, aStack, aPlayer);
    	}
        return aStack;
    }
    
	@Override
	public Long[] getElectricStats(ItemStack aStack) {
		return mElectricStats.get((short)aStack.getItemDamage());
	}
	
	@Override
	public Long[] getFluidContainerStats(ItemStack aStack) {
		return mFluidContainerStats.get((short)aStack.getItemDamage());
	}
	
	@Override
	@Optional.Method(modid = ModIDs.APC)
	public FoodValues getFoodValues(ItemStack aStack) {
    	IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
    	return tStat==null?null:new FoodValues(tStat.getFoodLevel(this, aStack, null), tStat.getSaturation(this, aStack, null));
	}
	
	@Override
    @SideOnly(Side.CLIENT)
    public void getSubItems(Item var1, CreativeTabs aCreativeTab, List aList) {
        for (int i = 0, j = mEnabledItems.length(); i < j; i++) if (mVisibleItems.get(i) || (SHOW_HIDDEN_ITEMS && mEnabledItems.get(i))) {
    		Long[] tStats = mElectricStats.get((short)i);
    		if (tStats != null && tStats[3] < 0) {
    			ItemStack tStack = new ItemStack(this, 1, i);
    			setCharge(tStack, Math.abs(tStats[0]));
    			isItemStackUsable(tStack);
            	aList.add(tStack);
    		}
    		if (tStats == null || tStats[3] != -2) {
    			ItemStack tStack = new ItemStack(this, 1, i);
    			isItemStackUsable(tStack);
            	aList.add(tStack);
    		}
        }
    }
	
	@Override
    @SideOnly(Side.CLIENT)
    public void registerIcons(IIconRegister aIconRegister) {
		for (short i = 0, j = (short)mEnabledItems.length(); i < j; i++) if (mEnabledItems.get(i)) {
			for (byte k = 1; k < mIconList[i].length; k++) {
				mIconList[i][k] = aIconRegister.registerIcon(RES_PATH_ITEM + getUnlocalizedName() + "/" + i + "/" + k);
			}
    		mIconList[i][0] = aIconRegister.registerIcon(RES_PATH_ITEM + getUnlocalizedName() + "/" + i);
    	}
    }
	
	@Override
    public IIcon getIconIndex(ItemStack aStack) {
		short aMetaData = (short)aStack.getItemDamage();
		if (!UT.Code.exists(aMetaData, mIconList)) return null;
		Long[] tStats = mElectricStats.get(aMetaData);
		if (tStats != null && tStats[3] < 0) {
			long tCharge = getRealCharge(aStack);
			if (tCharge <= 0) return mIconList[aMetaData][1];
			if (tCharge >= tStats[0]) return mIconList[aMetaData][8];
			return mIconList[aMetaData][7-(int)Math.max(0, Math.min(5, ((tStats[0]-tCharge)*6L) / tStats[0]))];
		}
		return mIconList[aMetaData][0];
    }
	
	@Override
    public IIcon getIconFromDamage(int aMetaData) {
		return UT.Code.exists(aMetaData, mIconList) ? mIconList[aMetaData][0] : null;
    }
	
	@Override
    public IIcon getIcon(ItemStack aStack, int aRenderPass) {
        return getIconIndex(aStack);
    }
	
	@Override
    public IIcon getIcon(ItemStack aStack, int aRenderPass, EntityPlayer aPlayer, ItemStack aUsedStack, int aUseRemaining) {
		return getIconIndex(aStack);
    }
	
    @Override
    public IIcon getIconFromDamageForRenderPass(int aMetaData, int aRenderPass) {
		return getIconFromDamage(aMetaData);
    }
}